#define S_FUNCTION_NAME lobdetection
#define S_FUNCTION_LEVEL 2

/* Author: Rene Vugts
 * Date: aug 2010 */

#include "simstruc.h"
#include "math.h"
#include "../../Global_par/constants.h"

/*******************
 ***** DEFINES *****
 *******************/
#define NUMBER_OF_SAMPLES           4       // 3 = MINIMUM!
#define MIN_RADIAL_VELOCITY_FIRST   2       // in m/s
#define MIN_RADIAL_VELOCITY_LAST    10      // in m/s

#ifndef dabs
#define dabs(A) ((A)>=0?(A):-(A))
#endif 

/*******************
 ***** STATICS *****
 *******************/
static bool   FB[NUMBER_OF_SAMPLES];
static double XB[NUMBER_OF_SAMPLES];
static double YB[NUMBER_OF_SAMPLES];

static double DR[NUMBER_OF_SAMPLES-1];
static double VR[NUMBER_OF_SAMPLES-1];

static void mdlInitializeSizes(SimStruct *S)
{
    /*PARAMETERS*/
    ssSetNumSFcnParams(S, 0);
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    
    /*INPUT PORTS*/
    if (!ssSetNumInputPorts(S, 2)) return;
    
    /* ov ball found [0/1] */
    ssSetInputPortWidth(S, 0, 1);
    ssSetInputPortDataType(S, 0, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortRequiredContiguous(S, 0, 1);
    
    /* ov ball local [xb,yb] */
    ssSetInputPortWidth(S, 1, 3);
    ssSetInputPortDataType(S, 1, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortRequiredContiguous(S, 1, 1);
       
       
    /*OUTPUT PORTS*/
    if (!ssSetNumOutputPorts(S, 1)) return;
    
    /* ball not found (-1), ball found but vr<vrmin or not computed (0), ball found and vr>vrmin (1), lob detected (2) */
    ssSetOutputPortWidth(S, 0, 1);
    ssSetOutputPortDataType(S, 0, SS_INT32);    
    
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

#define MDL_INITIALIZE_CONDITIONS
static void mdlInitializeConditions(SimStruct *S)
{
    int i;
    for (i=0;i<NUMBER_OF_SAMPLES;i++) {
        FB[i] = 0;
        XB[i] = 0.0;
        YB[i] = 0.0;
    }
    for (i=0;i<NUMBER_OF_SAMPLES-1;i++) {
        DR[i] = 0.0;
        VR[i] = 0.0;
    }
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
    /*inputs*/
    const real_T  *ovballconf   = (const real_T*)    ssGetInputPortSignal(S,0);
    const real_T *ovballpos     = (const real_T*)   ssGetInputPortSignal(S,1);

    /*outputs*/
    int_T        *lobdet         = (int_T*)         ssGetOutputPortSignal(S,0);
    
    /* other */
    bool    fb  = ovballconf[0]>0.5;
    double  xb  = ovballpos[0];
    double  yb  = ovballpos[1];
    
    int lob = 0;
    bool lobcheck = true;
    
    /* update statics (latest measurement on top (i=0))*/
    int i=0;
    for (i=NUMBER_OF_SAMPLES-1;i>0;i--) {
        FB[i] = FB[i-1];
        XB[i] = XB[i-1];
        YB[i] = YB[i-1];
    }
    FB[0] = fb;
    XB[0] = xb;
    YB[0] = yb;
        
    /***************************** main code *****************************/
    
    /* find the last samples for which the ball was found */
    int number_of_samples = 0;
    for (i=0;i<NUMBER_OF_SAMPLES;i++) {
        if (FB[i]) {
            number_of_samples += 1;
        }
        else {
            break;
        }
    }
    
    /* compute the velocity in radial direction */
    if (number_of_samples<2) {
        /* radial velocity cannot be computed */
        
        /* write outputs */
        if (!FB[0]) {
            /* ball not found */
            lob = -1;
        }
        else {
            /* ball found but not the previous sample */
            lob = 0;
        }
    }
    else {
        /* ball is found 2 or more samples */
        
        /* compute radial velocity */
        for (i=0;i<number_of_samples-1;i++) {
            DR[i] = sqrt(XB[i]*XB[i]+YB[i]*YB[i])-sqrt(XB[i+1]*XB[i+1]+YB[i+1]*YB[i+1]);
            VR[i] = DR[i]*VISION_SAMPLE_RATE;
        }
        
        if (!number_of_samples==NUMBER_OF_SAMPLES) {
            /* not all balls found */
            if (number_of_samples==2) {
                /* only one velocity measurement */
                lob = 0;
            }
            else {
                /* two or more velocity measurements */
                lobcheck = (VR[0]>VR[1]) && (VR[1]>MIN_RADIAL_VELOCITY_FIRST);
                if (lobcheck) {
                /* increasing radial velocity: lob danger */
                    lob = 1;
                }
                else {
                /* no lob danger */
                    lob = 0;
                }
            }
        }
        else {
            /* all balls found (number_of_samples==NUMBER_OF_SAMPLES) , do lob check */
            lobcheck = (VR[0]>MIN_RADIAL_VELOCITY_LAST);

            if (NUMBER_OF_SAMPLES==3) {
                /* lobdetection with 2 velocity samples */
                lobcheck = lobcheck && (VR[1]>MIN_RADIAL_VELOCITY_FIRST);
            }
            else {
                /* 4 or more position samples: 3 or more velocity samples available */
                for (i=1;i<NUMBER_OF_SAMPLES-2;i++) {
                    lobcheck = lobcheck && (VR[i]>VR[i+1]);
                }
                lobcheck = lobcheck && (VR[NUMBER_OF_SAMPLES-2]>MIN_RADIAL_VELOCITY_FIRST);
            }
            
            if (lobcheck) {
            /* lob ! */
                lob = 2;
            }
            else if ((VR[0]>VR[1]) && (VR[1]>MIN_RADIAL_VELOCITY_FIRST)) {
            /* lob danger */
                lob = 1;
            }
            else {
            /* no lob danger */
                lob = 0;
            }            
        }
    }
    
    /* write output */
    *lobdet = lob;    
}

#undef MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#undef MDL_UPDATE  /* Change to #undef to remove function */
#undef MDL_DERIVATIVES  /* Change to #undef to remove function */

static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
