%% @file
% tests the vision algorithms offline including color calibration, mask
% design, self localisation, obstacle detection, line points detection,
% obstacle avoidance
%
% usage: testvision(turtle)
%
% @author DJH Bruijnen <d.j.h.bruijnen@tue.nl>
% @date 2006
%
% @param turtle - turtle string-ID, valid inputs are "turtlex", 1<=x<=7 [char array, 1x7]
% @return none

function testvision(varargin)

try
    %init
    turtle=varargin{1};

    %copy calibration data to testvision directory and load
    copydata(turtle);
    [mask,mc,fm,fmx,fmy,fmn,heading_bluegoal] = loaddata;

    %% get image/sequence file
    [A, compass_data, isimage] = selectsequence(turtle);

    %setparameters
    setparametersmdl(A,compass_data,isimage,turtle);

    %simulate
    [NLP,LP,PV,ST,OBS,BALL,GOALS,LPR,Ymap,Umap,Vmap,FV,LABELS] = dosimulation;

    if isimage
        plotimage(A,compass_data,NLP,LP,PV,ST,OBS,BALL,GOALS,LPR,LABELS);
        title(['nlinepoints: ' int2str(NLP(end)) ', fitval: ' num2str(FV(end),'%1.3g')])
    else
        figure;
        subplot(221)
        plotstuff(0,LPR,PV,mod(pi+pi/180*-(compass_data-heading_bluegoal),2*pi)-pi);
        h1 = plot([1;1],[-10 10],'k');

        %plot linepoints
        subplot(224)
        title('fitness value, linepoints')
        plotstuff(7,NLP,FV)
        h2 = plot([1;1],[0;300],'k');

        %get data
        i=0;
        doloop = 1;
        PVsel = PV(find(sum(LPR,2)==2),:);
        while doloop>.5
            if doloop==1
                i = i+1;
                if i>=length(LPR)
                    doloop=2;
                end
            else
                [mousex,mousey,mouseb]=ginput(1);
                if mouseb==3
                    break
                elseif mouseb==2
                    doloop=1;
                end
                i = min(max(1,round(mousex)),length(LPR));
            end

            [nlinepoints,linepoints,posevision,subtarget,obstacles,ball,goals,labels]=getdata(i,NLP,LP,PV,ST,OBS,BALL,GOALS,LPR,LABELS);

            subplot(221)
            title(['pose_vision, frame: ' int2str(i)])
            set(h1,'xdata',[i;i])

            %plot global view
            subplot(222)
            plotstuff(3,mc,fm,fmx,fmy,fmn,linepoints,posevision,PVsel,obstacles,ball,goals);

            %plot top view
            subplot(223)
            map(:,:,1)=Ymap(:,:,i);
            map(:,:,2)=Umap(:,:,i);
            map(:,:,3)=Vmap(:,:,i);
            plotstuff(5,map,linepoints,obstacles,posevision,mc,evalin('base','target'),subtarget,ball,goals);
            set(h2,'xdata',[i;i])
            drawnow

        end
    end
    delete *.dat
    bdclose
catch
    closetool();
end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plotintopview(obj,posevision,mc,res,mmap,varargin)
obj_r = sqrt(sum((posevision(1:2)-obj).^2));
obj_phi = atan2(-obj(1)+posevision(1),obj(2)-posevision(2))-posevision(3)+mc(9);
obj_x = -obj_r*sin(obj_phi);
obj_y = obj_r*cos(obj_phi);
plot((mc(5)+obj_x)/res+mmap,(-mc(6)-obj_y)/res+mmap,varargin{:})

function plotobstacleintopview(obj,posevision,mc,res,mmap,varargin)
circle_x=obj(3)*sin(linspace(0,2*pi,50));
circle_y=obj(3)*cos(linspace(0,2*pi,50));
obj_r = sqrt(sum((posevision(1:2)-obj(1:2)').^2));
obj_phi = atan2(-obj(1)+posevision(1),obj(2)-posevision(2))-posevision(3)+mc(9);
obj_x = -obj_r*sin(obj_phi);
obj_y = obj_r*cos(obj_phi);
plot((mc(5)+obj_x+circle_x)/res+mmap,(-mc(6)-obj_y+circle_y)/res+mmap,varargin{:})

function mfilepath = gotomfilepath
mfilepath = mfilename('fullpath');
ind = [strfind(mfilepath,'/') strfind(mfilepath,'\')];
mfilepath=mfilepath(1:max(ind));
cd(mfilepath);

function copydata(turtle)
mfilepath = gotomfilepath;
cd ../..;
pathnamemask=['Calibration_data_' turtle];
cd(pathnamemask);
pathnamemask = [cd '/'];
copyfile('*.*',mfilepath,'f')

%copy fieldmap to testvision directory
cd ..;
copyfile('fieldmap.dat',mfilepath,'f');

function [mask,mc,fm,fmx,fmy,fmn,heading_bluegoal] = loaddata
gotomfilepath;

%mask data
fid=fopen('mask.dat','rb');
mask=fread(fid,[640 480],'*uint8');
fclose(fid);

%get mirror parameters
fid = fopen('mc.dat','rb');
mc = fread(fid,9,'double');
fclose(fid);

%get mirror parameters
fid = fopen('compass.dat','rb');
heading_bluegoal = fread(fid,1,'double');
fclose(fid);

%load fieldmap data
fieldwidth = get_global_par('FIELDWIDTH');
fieldlength = get_global_par('FIELDLENGTH');
resolution = get_global_par('FIELDMAP_RESOLUTION');
borderwidth = get_global_par('FIELDMAP_BORDERWIDTH');
fmx = fieldwidth/2+borderwidth;
fmy = fieldlength/2+borderwidth;
fmn = resolution;
fid = fopen('fieldmap.dat','rb');
fm = fread(fid,[fmx*fmn*2+1 fmy*fmn*2+1],'ubit8').';
fclose(fid);



function [A, compass_data,isimage] = selectsequence(turtle)
gotomfilepath;
cd ..;
cd capture_photo;

[filename, pathname] = uigetfile({'*.png;*.mat'}, 'Select an image or sequence for testing');

cd(pathname);
if strcmp(filename(end-2:end),'png'), %image selected
    isimage = 1;

    %load compass-data
    try
        load('-mat',[filename(1:end-3) 'cps']);
    catch
        try
            compass_data = load('-ASCII',[filename(1:end-3) 'cps']);
        catch
            errordlg(['unable to load compass data file ' filename(1:end-3) 'cps!'])
        end
    end

    %load image
    A=imread(filename);
elseif strcmp(filename(end-2:end),'mat'), %sequence selected
    isimage = 0;
    load(filename)
end
assignin('base','A',A);

function plotstuff(subplotnumber,varargin)

if subplotnumber==0
    cla 
    lineposrecog = varargin{1};
    posevision = varargin{2};
    compass_data = varargin{3};
    
    screensize = get(0,'screensize');
    set(gcf,'position',[screensize(3:4)/16 screensize(3:4)*(1-1/16*2)])
    
    ind1 = find(sum(lineposrecog,2)==0);
    ind2 = find(lineposrecog(:,1)==1 & lineposrecog(:,2)==0);
    ind3 = find(lineposrecog(:,1)==0 & lineposrecog(:,2)==1);
    ind4 = find(sum(lineposrecog,2)==2);


    hold on
    ylimits = [min([posevision(:); -pi]) max([posevision(:);pi])];
    if length(ind1)>0
        fill([ind1'+.5;ind1'+.5;ind1'-.5;ind1'-.5],ylimits([1 2 2 1])',[1 .8 .8],'edgecolor',[1 .8 .8])
    end
    if length(ind2)>0
        fill([ind2'+.5;ind2'+.5;ind2'-.5;ind2'-.5],ylimits([1 2 2 1])',[.8 .8 .8],'edgecolor',[.8 .8 .8])
    end
    if length(ind3)>0
        fill([ind3'+.5;ind3'+.5;ind3'-.5;ind3'-.5],ylimits([1 2 2 1])',[1 0 1],'edgecolor',[1 0 1])
    end
    if length(ind4)>0
        fill([ind4'+.5;ind4'+.5;ind4'-.5;ind4'-.5],ylimits([1 2 2 1])',[.8 1 .8],'edgecolor',[.8 1 .8])
    end
    
    plot([ind2;ind4],posevision([ind2;ind4],:),'*');
    plot(compass_data,'color',[.5 0 0])
    axis tight
    ylabel('pose')
    xlabel('image')
elseif subplotnumber==1
    cla
    A = varargin{1};
    mask = varargin{2};

    imshow(ycbcr2rgb(A) .* (repmat(mask,[1 1 3])/255))
    hold on
    title({'omni-vision image with mask: line-points (red stars),','pose-center (red cross), robot-center (blue cross)'})

    if length(varargin)>2
        mc = varargin{3};
        linepoints = varargin{4};

        %calculate linepoints in mirror image
        a = atan2(-linepoints(:,1),linepoints(:,2));
        d = sqrt(sum(linepoints.^2,2));
        d_pix = atan(d/mc(3))/mc(4);
        px = mc(1)-cos(a).*d_pix;
        py = mc(2)-sin(a).*d_pix;

        %plot stuff
        plot(py,px,'*r')
        plot(mc(2),mc(1),'xr')
        plot(mc(8),mc(7),'xb')
        plot([mc(8) mc(8)-sin(mc(9))*50],[mc(7) mc(7)-cos(mc(9))*50],'b')
    end

elseif subplotnumber==2
    cla
    map = varargin{1};
    labels = varargin{2};

    %get top view map
    mmap=(size(map,1)-1)/2+1;
    colors=[0 0 0;1 .5 0;0 .5 0;zeros(1,3);1 1 1;zeros(3,3);.5 .5 .5;zeros(7,3);0 0 1;zeros(15,3);1 1 0];
    kmin=min(labels(:));
    kmax=max(labels(:));
    caxis([0 kmax]);
    tmp = colors(1+(kmin:kmax),:);
    subimage(labels,tmp)
    title('color segmentation image')
    axis off
elseif subplotnumber==3
    cla
    mc = varargin{1};
    fm = varargin{2};
    fmx = varargin{3};
    fmy = varargin{4};
    fmn = varargin{5};
    linepoints = varargin{6};
    posevision = varargin{7};
    PV = varargin{8};
    obstacles = varargin{9};
    ball = varargin{10};
    goals = varargin{11};

    fieldlength = get_global_par('FIELDLENGTH');
    goalwidth = get_global_par('GOALWIDTH');

    %plot fieldmap
    cla
    imagesc(fm)
    axis image off
    colormap('gray')
    caxis([0 255])
    title(['pose (blue cross) = [' num2str(posevision) ']'])

    %plot estimated position
    hold on
    if size(PV,1)>0
        plotinglobal(PV(:,1:2),fmx,fmy,fmn,'.','color',[0 .5 0],'markersize',3);
    end
    plotinglobal(posevision(1:2),fmx,fmy,fmn,'bx','linewidth',1)
    plot((posevision(1)+[0 -.5*sin(posevision(3))]+fmx)*fmn,(-posevision(2)+[0 -.5*cos(posevision(3))]+fmy)*fmn,'b','linewidth',1)

    %plot position used for self localisation
    posx = posevision(1)-cos(posevision(3)-mc(9))*mc(5)+sin(posevision(3)-mc(9))*mc(6);
    posy = posevision(2)-sin(posevision(3)-mc(9))*mc(5)-cos(posevision(3)-mc(9))*mc(6);
%     plot((posx+fmx)*fmn,(-posy+fmy)*fmn,'rx','linewidth',1)

    %plot linepoints in field
    lpgx = posx+cos(posevision(3)-mc(9))*linepoints(:,1)-sin(posevision(3)-mc(9))*linepoints(:,2);
    lpgy = posy+sin(posevision(3)-mc(9))*linepoints(:,1)+cos(posevision(3)-mc(9))*linepoints(:,2);
    plot((lpgx+fmx)*fmn,(-lpgy+fmy)*fmn,'r*')

    %plot goal locations
    if evalin('base','blueishomegoal')
        plot(([-1 1]*goalwidth/2+fmx)*fmn,([1 1]*fieldlength/2+fmy)*fmn,'-b','linewidth',5)
        plot(([-1 1]*goalwidth/2+fmx)*fmn,([1 1]*-fieldlength/2+fmy)*fmn,'-y','linewidth',5)
    else
        plot(([-1 1]*goalwidth/2+fmx)*fmn,([1 1]*-fieldlength/2+fmy)*fmn,'-b','linewidth',5)
        plot(([-1 1]*goalwidth/2+fmx)*fmn,([1 1]*fieldlength/2+fmy)*fmn,'-y','linewidth',5)
    end
    
    %plot ball
    plotinglobal(ball,fmx,fmy,fmn,'o','color',[1 .5 0],'linewidth',3);
    
    %plot obstacles
    phi = linspace(0,2*pi,20)';
    for i=1:size(obstacles,2)
        plot((obstacles(1,i)+fmx)*fmn+cos(phi)*obstacles(3,i)*fmn,...
            (obstacles(2,i)+fmy)*fmn+sin(phi)*obstacles(3,i)*fmn,'m')
    end
    
    %plot goals
    plotinglobal(goals(1:2),fmx,fmy,fmn,'x','color',[0 0 .5],'linewidth',3)
    plotinglobal(goals(3:4),fmx,fmy,fmn,'x','color',[.5 .5 0],'linewidth',3)
    
elseif subplotnumber==4
    cla

    posetime = varargin{1};

    plot(posetime,'o')
    ind=find(posetime(:,1)~=posetime(1,1));
    xlabel('iterations')
    ylabel('pose')
    if ~isempty(ind)
        hold on
        plot([ind(1) ind(1)],ylim,'k')
        title(['pose found after ' int2str(ind(1)) ' iterations'])
    else
        title('pose not found!')
    end
    xlim([0 size(posetime,1)])

elseif subplotnumber==5
    cla

    map = varargin{1};
    linepoints = varargin{2};
    obstacles = varargin{3};
    posevision = varargin{4};
    mc = varargin{5};
    target = varargin{6};
    subtarget = varargin{7};
    ball = varargin{8};
    goals = varargin{9};

    %get top view map
    mmap=(size(map,1)-1)/2+1;
    res=0.05;
    imshow(ycbcr2rgb(map))
    title({'top view: obstacles (purple), ball (orange), subtarget (cyan cross)',['target (cyan circle) = [' num2str(target) ']']})

    %plot linepoints in top view
    hold on
    plot(linepoints(:,1)/res+mmap,-linepoints(:,2)/res+mmap,'r*')

    %plot obstacles
    for i=1:size(obstacles,2)
        plotobstacleintopview(obstacles(:,i),posevision,mc,res,mmap,'m')
    end

    %plot target
    plotintopview(target,posevision,mc,res,mmap,'co')

    %plot subtarget
    plotintopview(subtarget,posevision,mc,res,mmap,'cx')

    %plot ball
    plotintopview(ball,posevision,mc,res,mmap,'o','color',[1 .5 0],'linewidth',1)

    %plot ball
    plotintopview(goals(1:2),posevision,mc,res,mmap,'x','color',[0 0 1],'linewidth',3)
    plotintopview(goals(3:4),posevision,mc,res,mmap,'x','color',[1 1 0],'linewidth',3)
elseif subplotnumber==6
    cla

    lineposrecog = varargin{1};

    plot(lineposrecog(:,1),'o')
    hold on
    plot(lineposrecog(:,2),'g-')
    %legend('line recog','pose recog')
    aar=axis;
    hoekx = [aar(2)-0.1*aar(2), aar(2)-0.1*aar(2), aar(2), aar(2)];
    hoeky = [aar(4)-0.1*aar(4), aar(4), aar(4), aar(4)-0.1*aar(4)];
    line([hoekx,hoekx(1)],[hoeky,hoeky(1)]);
    if lineposrecog(end,1) && lineposrecog(end,2)
        fill(hoekx,hoeky,'g')
    elseif lineposrecog(end,1)
        fill(hoekx,hoeky,'w')
    elseif lineposrecog(end,2)
        fill(hoekx,hoeky,'m')
    else
        fill(hoekx,hoeky,'r')
    end
elseif subplotnumber == 7
    cla
    nlinepoints = varargin{1};
    fitval = varargin{2};
    
    h=plotyy(1:length(nlinepoints),fitval,1:length(nlinepoints),nlinepoints);
    hold(h(1),'on')
    hold(h(2),'on')
    set(h,'xlim',[0 length(nlinepoints)])
    ylabel(h(1),'fitness value')
    ylabel(h(2),'linepoints')
    xlabel('image')
    fitvalthr = get_global_par('WHITEPOINTS_ALLOWEDERROR');
    plot([1 length(nlinepoints)],[fitvalthr fitvalthr],':b')
end

function plotinglobal(obj,fmx,fmy,fmn,varargin)
objx = (obj(:,1)+fmx)*fmn;
objy = (-obj(:,2)+fmy)*fmn;
plot(objx,objy,varargin{:})


function setparametersmdl(A,compass_data,isimage,turtle)

gotomfilepath;

%home goal
blueishomegoal = 0;
evalin('base',['blueishomegoal=' int2str(blueishomegoal) ';'])

%% get linepoints
target = [-2 2];
evalin('base',['target = [' num2str(target) '];'])
cd ../..;
init_vision(turtle);
gotomfilepath;

%simulate once again
if isimage
    A(:,:,:,2)=A(:,:,:,1);
    assignin('base','compass_heading',[0 compass_data]);
    assignin('base','A',A);
    load_system('testvisionscheme')
    set_param('testvisionscheme/outputmap','Value','0')
else
    assignin('base','compass_heading',[[0:length(compass_data)-1]' compass_data]);
    load_system('testvisionscheme')
    set_param('testvisionscheme/sequence','Signal','A')
    set_param('testvisionscheme/outputmap','Value','1')
end

function [map,labels] = getmap(i)
evalin('base','T = 0;')
k = get_param('testvisionscheme/sequence','Signal');
set_param('testvisionscheme/sequence','Signal',['A(:,:,:,[' int2str([i 1]) '])'])
set_param('testvisionscheme/outputmap','Value','1')
warning off %warnings are thrown if blob analysis reaches the maximum amount which is time consuming
sim('testvisionscheme')
warning on
set_param('testvisionscheme/outputmap','Value','0')
set_param('testvisionscheme/sequence','Signal',k)

map(:,:,1) = Ymap;
map(:,:,2) = Umap;
map(:,:,3) = Vmap;


function [nlinepoints,linepoints,posevision,subtarget,obstacles,ball,goals,lineposrecog,Ymap,Umap,Vmap,fitval,labels] = dosimulation
evalin('base','T=max(20,size(compass_heading,1));')
evalin('base','safetydistance = 0.25;')
% evalin('base','subtargetsaturation = .5*[get_global_par(''FIELDWIDTH'') get_global_par(''FIELDLENGTH'')]+0.5;')
% evalin('base','subtargetsaturation_alltasks = .5*[get_global_par(''FIELDWIDTH'') get_global_par(''FIELDLENGTH'')]+1.0;')
% evalin('base','par_determinesubtarget = [safetydistance];')
warning off
sim('testvisionscheme');
warning on

function [nlinepoints,linepoints,posevision,subtarget,obstacles,ball,goals,labels]=getdata(i,NLP,LP,PV,ST,OBS,BALL,GOALS,LPR,LABELS)
nlinepoints = NLP(i);%evalin('base','nlinepoints(2);')
linepoints = LP(1:nlinepoints,:,i);%evalin('base','linepoints(1:nlinepoints,:,2)+1;') %+1 because indices come from C en start with 0
posevision = PV(i,:);
subtarget = ST(i,:);
obstacles = reshape(OBS(i,:),3,length(OBS(i,:))/3);
ind=find(obstacles(3,:)==0);
if ~isempty(ind)
    obstacles =obstacles(:,1:ind-1);
end
ball = BALL(i,:);
goals = GOALS(i,:);
labels = LABELS(:,:,i);

function plotimage(A,compass_data,NLP,LP,PV,ST,OBS,BALL,GOALS,LPR,labels)

%select data
[nlinepoints,linepoints,posevision,subtarget,obstacles,ball,goals,labels]=getdata(length(NLP),NLP,LP,PV,ST,OBS,BALL,GOALS,LPR,labels);
[mask,mc,fm,fmx,fmy,fmn,heading_bluegoal] = loaddata;
[map,labelsss]=getmap(1);
mmap=(size(map,1)-1)/2+1;
colors=[0 0 0;1 .5 0;0 .5 0;zeros(1,3);1 1 1;zeros(3,3);.5 .5 .5;zeros(7,3);0 0 1;zeros(15,3);1 1 0];
res=0.05;
target=evalin('base','target');

%create figure
h_testvision=figure;
screensize = get(0,'screensize');
set(h_testvision,'position',[screensize(3:4)/16 screensize(3:4)*(1-1/16*2)])
set(h_testvision,'deletefcn',@closetool);

%plot omni image
subplot(231)
imshow(ycbcr2rgb(A) .* (repmat(mask,[1 1 3])/255))
hold on
title({'omni-vision image with mask: line-points (red stars),','pose-center (red cross), robot-center (blue cross)'})

%calculate linepoints in mirror image
a = atan2(-linepoints(:,1),linepoints(:,2));
d = sqrt(sum(linepoints.^2,2));
d_pix = atan(d/mc(3))/mc(4);
px = mc(1)-cos(a).*d_pix;
py = mc(2)-sin(a).*d_pix;

%plot stuff
plot(py+1,px+1,'*r')
plot(mc(2)+1,mc(1)+1,'xr')
plot(mc(8)+1,mc(7)+1,'xb')
plot([mc(8)+1 mc(8)+1-sin(mc(9))*50],[mc(7)+1 mc(7)+1-cos(mc(9))*50],'b')

%plot top view
subplot(232)

%get top view map
kmin=double(min(labels(:)));
kmax=double(max(labels(:)));
labels = uint8(labels);
caxis([0 kmax]);
tmp = colors(1+(kmin:kmax),:);
subimage(labels,tmp)
title('color segmentation image')
axis off

%plot linepoints
subplot(233)

fieldlength = get_global_par('FIELDLENGTH');
goalwidth = get_global_par('GOALWIDTH');

%plot fieldmap
cla
imagesc(fm)
axis image off
colormap('gray')
caxis([0 255])
title(['pose (blue cross) = [' num2str(posevision) ']'])

%plot estimated position
hold on
plot((posevision(1)+fmx)*fmn,(-posevision(2)+fmy)*fmn,'bx','linewidth',1)
plot((posevision(1)+[0 -.5*sin(posevision(3))]+fmx)*fmn,(-posevision(2)+[0 -.5*cos(posevision(3))]+fmy)*fmn,'b','linewidth',1)

%plot position used for self localisation
posx = posevision(1)-cos(posevision(3)-mc(9))*mc(5)+sin(posevision(3)-mc(9))*mc(6);
posy = posevision(2)-sin(posevision(3)-mc(9))*mc(5)-cos(posevision(3)-mc(9))*mc(6);
plot((posx+fmx)*fmn,(-posy+fmy)*fmn,'rx','linewidth',1)

%plot linepoints in field
lpgx = posx+cos(posevision(3)-mc(9))*linepoints(:,1)-sin(posevision(3)-mc(9))*linepoints(:,2);
lpgy = posy+sin(posevision(3)-mc(9))*linepoints(:,1)+cos(posevision(3)-mc(9))*linepoints(:,2);
plot((lpgx+fmx)*fmn,(-lpgy+fmy)*fmn,'r*')

%plot goal locations
if evalin('base','blueishomegoal')
plot(([-1 1]*goalwidth/2+fmx)*fmn,([1 1]*fieldlength/2+fmy)*fmn,'-b','linewidth',5)
plot(([-1 1]*goalwidth/2+fmx)*fmn,([1 1]*-fieldlength/2+fmy)*fmn,'-y','linewidth',5)
else
    plot(([-1 1]*goalwidth/2+fmx)*fmn,([1 1]*-fieldlength/2+fmy)*fmn,'-b','linewidth',5)
    plot(([-1 1]*goalwidth/2+fmx)*fmn,([1 1]*fieldlength/2+fmy)*fmn,'-y','linewidth',5)
end

%plot pose
subplot(234)

plot(PV,'o')
ind=find(PV(:,1)~=PV(1,1));
xlabel('iterations')
ylabel('pose')
if ~isempty(ind)
    hold on
    plot([ind(1) ind(1)],ylim,'k')
    title(['pose found after ' int2str(ind(1)) ' iterations'])
else
    title('pose not found!')
end
xlim([0 size(PV,1)])

%plot top view
subplot(235)
imshow(ycbcr2rgb(map))
title({'top view: obstacles (purple), ball (orange), subtarget (cyan cross)',['target (cyan circle) = [' num2str(target) ']']})

%plot linepoints in top view
hold on
plot(linepoints(:,1)/res+mmap,-linepoints(:,2)/res+mmap,'r*')

%plot obstacles
for i=1:size(obstacles,2)
    plotobstacleintopview(obstacles(:,i),posevision,mc,res,mmap,'m')
end

%plot target
plotintopview(target,posevision,mc,res,mmap,'co')

%plot subtarget
plotintopview(double(subtarget),posevision,mc,res,mmap,'cx')

%plot ball
plotintopview(ball(1:2),posevision,mc,res,mmap,'o','color',[1 .5 0],'linewidth',1)

%plot recog
subplot(236)

plot(LPR(:,1),'o')
hold on
plot(LPR(:,2),'g-')
%legend('line recog','pose recog')
aar=axis;
hoekx = [aar(2)-0.1*aar(2), aar(2)-0.1*aar(2), aar(2), aar(2)];
hoeky = [aar(4)-0.1*aar(4), aar(4), aar(4), aar(4)-0.1*aar(4)];
line([hoekx,hoekx(1)],[hoeky,hoeky(1)]);
if LPR(end,1) && LPR(end,2)
    fill(hoekx,hoeky,'g')
elseif LPR(end,1)
    fill(hoekx,hoeky,'w')
elseif LPR(end,2)
    fill(hoekx,hoeky,'m')
else
    fill(hoekx,hoeky,'r')
end
